=== added directory 'state'
=== added file 'state/Makefile'
--- state/Makefile	1970-01-01 00:00:00 +0000
+++ state/Makefile	2011-12-20 10:52:07 +0000
@@ -0,0 +1,25 @@
+include $(GOROOT)/src/Make.inc
+
+all: package
+
+TARG=launchpad.net/juju/go/state
+
+GOFILES=\
+	state.go\
+	service.go\
+	topology.go\
+
+GOFMT=gofmt
+BADFMT:=$(shell $(GOFMT) -l $(GOFILES) $(CGOFILES) $(wildcard *_test.go))
+
+gofmt: $(BADFMT)
+	@for F in $(BADFMT); do $(GOFMT) -w $$F && echo $$F; done
+
+ifneq ($(BADFMT),)
+ifneq ($(MAKECMDGOALS),gofmt)
+$(warning WARNING: make gofmt: $(BADFMT))
+endif
+endif
+
+include $(GOROOT)/src/Make.pkg
+

=== added file 'state/doc.go'
--- state/doc.go	1970-01-01 00:00:00 +0000
+++ state/doc.go	2011-12-20 10:52:07 +0000
@@ -0,0 +1,14 @@
+// launchpad.net/juju/state
+//
+// Copyright (c) 2011 Canonical Ltd.
+
+/*
+ The state package monitor reads and monitors the
+ state data stored in the passed ZooKeeper connection.
+ The access to the topology is via the topology type,
+ a concurrently working manager which is updated
+ automatically using a gozk watch.
+*/
+package state
+
+// EOF

=== added file 'state/service.go'
--- state/service.go	1970-01-01 00:00:00 +0000
+++ state/service.go	2011-12-20 10:52:07 +0000
@@ -0,0 +1,69 @@
+// launchpad.net/juju/state
+//
+// Copyright (c) 2011 Canonical Ltd.
+
+package state
+
+// --------------------
+// IMPORT
+// --------------------
+
+import (
+	"fmt"
+)
+
+// --------------------
+// SERVICE
+// --------------------
+
+// Service represents the state of one service and
+// provides also the access to the subordinate parts.
+type Service struct {
+	topology *topology
+	id       string
+	name     string
+}
+
+func newService(t *topology, id, name string) *Service {
+	return &Service{t, id, name}
+}
+
+// Id returns the service id.
+func (s Service) Id() string {
+	return s.id
+}
+
+// Name returns the service name.
+func (s Service) Name() string {
+	return s.name
+}
+
+// CharmId returns the charm id this service is supposed to use.
+func (s Service) CharmId() string {
+	cid, err := s.topology.getString(s.zookeeperPath() + "/charm")
+
+	if err != nil {
+		// TODO: Ouch! Error handling.
+		panic("TODO: Error handling!")
+	}
+
+	return cid
+}
+
+// zookeeperPath returns the path within ZooKeeper.
+func (s Service) zookeeperPath() string {
+	return fmt.Sprintf("/services/%s", s.id)
+}
+
+// configPath returns the ZooKeeper path to the configuration.
+func (s Service) configPath() string {
+	return fmt.Sprintf("%s/config", s.zookeeperPath())
+}
+
+// exposedPath, if exists in ZooKeeper, indicates, that a
+// service is exposed.
+func (s Service) exposedPath() string {
+	return fmt.Sprintf("/services/%s/exposed", s.id)
+}
+
+// EOF

=== added file 'state/state.go'
--- state/state.go	1970-01-01 00:00:00 +0000
+++ state/state.go	2011-12-20 10:52:07 +0000
@@ -0,0 +1,71 @@
+// launchpad.net/juju/state
+//
+// Copyright (c) 2011 Canonical Ltd.
+
+package state
+
+// --------------------
+// IMPORT
+// --------------------
+
+import (
+	"errors"
+	"launchpad.net/gozk/zookeeper"
+)
+
+// --------------------
+// CONST
+// --------------------
+
+// --------------------
+// STATE
+// --------------------
+
+// State is the entry point to get access to the states
+// of the parts of the managed environmens.
+type State struct {
+	topology *topology
+}
+
+// Open returns a new or already created instance of
+// the state.
+func Open(zkc *zookeeper.Conn) (*State, error) {
+	t, err := retrieveTopology(zkc)
+
+	if err != nil {
+		return nil, errors.New("state: " + err.Error())
+	}
+
+	return &State{
+		topology: t,
+	}, nil
+}
+
+// Service returns a service state by name.
+func (s *State) Service(serviceName string) (*Service, error) {
+	// Search inside topology.
+	result, err := s.topology.execute(func(tn topologyNodes) (interface{}, error) {
+		spath, _, serr := tn.search(func(p []string, v interface{}) bool {
+			if len(p) == 3 && p[0] == "services" && p[len(p)-1] == "name" && v == serviceName {
+				return true
+			}
+
+			return false
+		})
+
+		if serr != nil {
+			return "", serr
+		}
+
+		return spath[1], nil
+	})
+
+	// Check the result of the command.
+	if err != nil {
+		return nil, err
+	}
+
+	return newService(s.topology, result.(string), serviceName), nil
+}
+
+// EOF

=== added file 'state/state_test.go'
--- state/state_test.go	1970-01-01 00:00:00 +0000
+++ state/state_test.go	2011-12-20 10:52:07 +0000
@@ -0,0 +1,253 @@
+// launchpad.net/juju/go/state
+//
+// Copyright (c) 2011 Canonical Ltd.
+
+package state
+
+// --------------------
+// IMPORT
+// --------------------
+
+import (
+	"errors"
+	"fmt"
+	. "launchpad.net/gocheck"
+	"launchpad.net/goyaml"
+	"launchpad.net/gozk/zookeeper"
+	"testing"
+	"time"
+)
+
+// --------------------
+// TEST DATA
+// --------------------
+
+// testTopology is a topology as YAML string.
+const testTopology = `
+services:
+    s-0:
+        name: service-zero
+        charm: my-charm-zero
+        units:
+            u-0:
+                sequence: 0
+            u-1:
+                sequence: 1
+    s-1:
+        name: service-one
+        charm: my-charm-one
+        units:
+    s-2:
+        name: service-two
+        charm: my-charm-two
+        units:
+unit-sequence:
+    service-zero: 2
+    service-one: 0
+    service-two: 0
+`
+
+// unmarshalTestTopology returns the testTopology as
+// topologyNodes.
+func unmarshalTestTopology(c *C) topologyNodes {
+	tn := newTopologyNodes()
+
+	if err := goyaml.Unmarshal([]byte(testTopology), &tn); err != nil {
+		c.Fail()
+
+		return nil
+	}
+
+	return tn
+}
+
+// setTestTopology sets the topology nodes in ZooKeeper.
+func setTestTopology(zkc *zookeeper.Conn, c *C) {
+	_, err := zkc.Create("/topology", testTopology, zookeeper.EPHEMERAL, zookeeper.WorldACL(zookeeper.PERM_ALL))
+
+	if err != nil {
+		c.Fatal("Cannot set test topology in ZooKeeper: ", err)
+	}
+}
+
+// --------------------
+// TESTING
+// --------------------
+
+func TestPackage(t *testing.T) {
+	TestingT(t)
+}
+
+// --------------------
+// TOPOLOGY NODE
+// --------------------
+
+// TopologyNodeSuite for topologyNode.
+type TopologyNodeSuite struct{}
+
+var _ = Suite(&TopologyNodeSuite{})
+
+// TestFind tests the find method of the topologyNodes.
+func (tn TopologyNodeSuite) TestFind(c *C) {
+	n := unmarshalTestTopology(c)
+
+	var data interface{}
+	var err error
+
+	data, err = n.find([]string{"services", "s-0", "name"})
+
+	c.Assert(data, Equals, "service-zero")
+	c.Assert(err, IsNil)
+
+	data, err = n.find([]string{"services", "s-0"})
+
+	c.Assert(data, FitsTypeOf, newTopologyNodes())
+	c.Assert(err, IsNil)
+
+	data, err = n.find([]string{"unknown", "path", "with", "no", "result"})
+
+	c.Assert(data, IsNil)
+	c.Assert(err, FitsTypeOf, errors.New(""))
+}
+
+// TestGetString tests the getString method of the topologyNodes.
+func (tn TopologyNodeSuite) TestGetString(c *C) {
+	n := unmarshalTestTopology(c)
+
+	var data string
+	var err error
+
+	data, err = n.getString("/services/s-0/name/")
+
+	c.Assert(data, Equals, "service-zero")
+	c.Assert(err, IsNil)
+
+	data, err = n.getString("/services/s-0")
+
+	c.Assert(data, Equals, "")
+	c.Assert(err, FitsTypeOf, errors.New(""))
+
+	data, err = n.getString("/unknown/path/with/no/result")
+
+	c.Assert(data, Equals, "")
+	c.Assert(err, FitsTypeOf, errors.New(""))
+}
+
+// TestGetNodes tests the getNodes method of the topologyNodes.
+func (tn TopologyNodeSuite) TestGetNodes(c *C) {
+	n := unmarshalTestTopology(c)
+
+	var data topologyNodes
+	var err error
+
+	data, err = n.getNodes("/services/s-0/name/")
+
+	c.Assert(data, IsNil)
+	c.Assert(err, FitsTypeOf, errors.New(""))
+
+	data, err = n.getNodes("/services/s-0")
+
+	c.Assert(data, FitsTypeOf, newTopologyNodes())
+	c.Assert(err, IsNil)
+
+	data, err = n.getNodes("/unknown/path/with/no/result")
+
+	c.Assert(data, IsNil)
+	c.Assert(err, FitsTypeOf, errors.New(""))
+}
+
+// TestSearch tests the search method of the topologyNodes.
+func (tn TopologyNodeSuite) TestSearch(c *C) {
+	n := unmarshalTestTopology(c)
+
+	path, value, err := n.search(func(p []string, v interface{}) bool {
+		if len(p) == 3 && p[0] == "services" && p[len(p)-1] == "name" && v == "service-one" {
+			return true
+		}
+
+		return false
+	})
+
+	c.Assert(err, IsNil)
+	c.Assert(path[1], Equals, "s-1")
+	c.Assert(value, Not(IsNil))
+}
+
+// --------------------
+// STATE
+// --------------------
+
+// StateSuite for State and the related types.
+type StateSuite struct {
+	zkServer   *zookeeper.Server
+	zkConn     *zookeeper.Conn
+	zkTestRoot string
+	zkTestPort int
+	zkAddr     string
+}
+
+var _ = Suite(&StateSuite{})
+
+// SetUpSuite starts and inits ZooKeeper.
+func (s *StateSuite) SetUpSuite(c *C) {
+	var err error
+
+	// Start server.
+	s.zkTestRoot = c.MkDir() + "/zookeeper"
+	s.zkTestPort = 21812
+	s.zkAddr = fmt.Sprint("localhost:", s.zkTestPort)
+
+	s.zkServer, err = zookeeper.CreateServer(s.zkTestPort, s.zkTestRoot, "")
+
+	if err != nil {
+		c.Fatal("Cannot set up ZooKeeper server environment: ", err)
+	}
+
+	err = s.zkServer.Start()
+
+	if err != nil {
+		c.Fatal("Cannot start ZooKeeper server: ", err)
+	}
+
+	// Establish connection after around a minute."
+	time.Sleep(60e9)
+
+	s.zkConn, _, err = zookeeper.Dial(s.zkAddr, 5e9)
+
+	if err != nil {
+		c.Fatal("Cannot establish ZooKeeper connection: ", err)
+	}
+
+	setTestTopology(s.zkConn, c)
+}
+
+// TearDownSuite stops ZooKeeper.
+func (s *StateSuite) TearDownSuite(c *C) {
+	if s.zkServer != nil {
+		if s.zkConn != nil {
+			// s.zkConn.Close()
+		}
+
+		s.zkServer.Destroy()
+	}
+}
+
+// TestService tests the Service  method of the State.
+func (s StateSuite) TestService(c *C) {
+	var err error
+	var state *State
+	var service *Service
+
+	state, err = Open(s.zkConn)
+
+	c.Assert(err, IsNil)
+	c.Assert(state, Not(IsNil))
+
+	service, err = state.Service("service-one")
+
+	c.Assert(err, IsNil)
+	c.Assert(service, Not(IsNil))
+	c.Assert(service.CharmId(), Equals, "my-charm-one")
+}
+
+// EOF

=== added file 'state/topology.go'
--- state/topology.go	1970-01-01 00:00:00 +0000
+++ state/topology.go	2011-12-20 10:52:07 +0000
@@ -0,0 +1,322 @@
+// launchpad.net/juju/state
+//
+// Copyright (c) 2011 Canonical Ltd.
+
+package state
+
+// --------------------
+// IMPORT
+// --------------------
+
+import (
+	"errors"
+	"launchpad.net/goyaml"
+	"launchpad.net/gozk/zookeeper"
+	"strings"
+)
+
+// --------------------
+// CONST
+// --------------------
+
+const (
+	Version = 1
+)
+
+// --------------------
+// GLOBAL VARIABLES
+// --------------------
+
+// systemTopology is the global topology that will be
+// initialized with the first call of retrieveTopology().
+// Then it will be reused and updated automatically after
+// signals via the watch.
+var systemTopology *topology
+
+// --------------------
+// TOPOLOGY NODES
+// --------------------
+
+// topologyNodes represents a set of nodes of the
+// topology informations.
+type topologyNodes map[interface{}]interface{}
+
+// newTopologyNodes creates node set with the version set.
+func newTopologyNodes() topologyNodes {
+	tn := make(topologyNodes)
+
+	tn["version"] = Version
+
+	return tn
+}
+
+// find looks for a value (a topologyNode or its end value) by
+// a given path. This path is a slice of strings, the end criteria
+// for this recursive call is an empty path. Here the node will
+// be returned.
+func (tn topologyNodes) find(path []string) (interface{}, error) {
+	if len(path) == 0 {
+		return tn, nil
+	}
+
+	value, ok := tn[path[0]]
+
+	if !ok {
+		// Not found!
+		return nil, errors.New("topology nodes: node '" + path[0] + "' not found")
+	}
+
+	if v, ok := value.(map[interface{}]interface{}); ok {
+		// More nodes.
+		vtn := topologyNodes(v)
+
+		return vtn.find(path[1:])
+	}
+
+	return value, nil
+}
+
+// getString retrieves the string value of a path. If it's a node
+// an error will be returned. The path will be passed as a string
+// with slashes as separators.
+func (tn topologyNodes) getString(path string) (string, error) {
+	value, err := tn.find(pathToSlice(path))
+
+	if err != nil {
+		return "", err
+	}
+
+	if v, ok := value.(string); ok {
+		// It's a string, yeah.
+		return v, nil
+	}
+
+	return "", errors.New("topology nodes: path '" + path + "' leads to topology nodes, no string")
+}
+
+// getNodes retrieves the nodes value of a path. If it's a string
+// an error will be returned. The path will be passed as a string
+// with slashes as separators.
+func (tn topologyNodes) getNodes(path string) (topologyNodes, error) {
+	value, err := tn.find(pathToSlice(path))
+
+	if err != nil {
+		return nil, err
+	}
+
+	if v, ok := value.(topologyNodes); ok {
+		// It's a topologyNodes, got it.
+		return v, nil
+	}
+
+	return nil, errors.New("topology nodes: path '" + path + "' leads to a string, no topology nodes")
+}
+
+// searchFunc defines the signature of a function for searches inside the topology nodes.
+// The arguments are the current path and the value. It has to return true if the search
+// matches.
+type searchFunc func(path []string, value interface{}) bool
+
+// search executes a search function recursively on the topology nodes. If this
+// function returns true the full path and its value will be returned.
+func (tn topologyNodes) search(sf searchFunc) ([]string, interface{}, error) {
+	path, value := tn.pathSearch([]string{}, sf)
+
+	if len(path) == 0 {
+		// Nothing found!
+		return nil, nil, errors.New("topology nodes: search has no results")
+	}
+
+	// Success, yay!
+	return path, value, nil
+}
+
+// pathSearch is used by search and has the current path as argument.
+func (tn topologyNodes) pathSearch(path []string, sf searchFunc) ([]string, interface{}) {
+	for key, value := range tn {
+		p := append(path, key.(string))
+
+		if sf(p, value) {
+			// Found it.
+			return p, value
+		}
+
+		if v, ok := value.(map[interface{}]interface{}); ok {
+			// Search not yet ok, but value is a topology node.
+			vtn := topologyNodes(v)
+			dp, dv := vtn.pathSearch(p, sf)
+
+			if len(dp) > 0 {
+				return dp, dv
+			}
+		}
+	}
+
+	// Found nothing.
+	return []string{}, nil
+}
+
+// pathToSlice converts a path string into a slice of strings.
+func pathToSlice(path string) []string {
+	pathSlice := strings.Split(path, "/")
+	cleanPathSlice := []string{}
+
+	for _, ps := range pathSlice {
+		if len(ps) > 0 {
+			cleanPathSlice = append(cleanPathSlice, ps)
+		}
+	}
+
+	return cleanPathSlice
+}
+
+// --------------------
+// COMMAND
+// --------------------
+
+// commandFunc is a function that will be sent to the backend
+// to analyze the topology nodes. It can return any result.
+type commandFunc func(topologyNodes) (interface{}, error)
+
+// commandResult encapsulates the result and an error for
+// transportation over a channel.
+type commandResult struct {
+	result interface{}
+	err    error
+}
+
+// command will be sent to the backend goroutine to perform
+// the task and return the answer.
+type command struct {
+	task       commandFunc
+	resultChan chan *commandResult
+}
+
+// newCommand creates a new command with the given task
+// and a new channel for the result.
+func newCommand(cf commandFunc) *command {
+	return &command{cf, make(chan *commandResult)}
+}
+
+// perform performs the task and returns the result
+// via the channel.
+func (c *command) perform(tn topologyNodes) {
+	result, err := c.task(tn)
+
+	c.resultChan <- &commandResult{result, err}
+}
+
+// --------------------
+// TOPOLOGY
+// --------------------
+
+// topology is an internal helper handling the topology informations
+// inside of ZooKeeper.
+type topology struct {
+	zkConn      *zookeeper.Conn
+	zkEventChan <-chan zookeeper.Event
+	commandChan chan *command
+	nodes       topologyNodes
+}
+
+// retrieveTopology connects ZooKeeper, retrieves the data as YAML,
+// parses and stores it.
+func retrieveTopology(zkc *zookeeper.Conn) (*topology, error) {
+	if systemTopology == nil {
+		systemTopology = &topology{
+			zkConn:      zkc,
+			commandChan: make(chan *command),
+			nodes:       newTopologyNodes(),
+		}
+
+		data, _, session, err := zkc.GetW("/topology")
+
+		if err != nil {
+			return nil, err
+		}
+
+		if err = goyaml.Unmarshal([]byte(data), systemTopology.nodes); err != nil {
+			return nil, err
+		}
+
+		systemTopology.zkEventChan = session
+
+		go systemTopology.backend()
+	}
+
+	return systemTopology, nil
+}
+
+// getString retrieves the string value of a path. If it's a node
+// an error will be returned. The path will be passed as a string
+// with slashes as separators.
+func (t *topology) getString(path string) (string, error) {
+	value, err := t.execute(func(tn topologyNodes) (interface{}, error) {
+		return tn.getString(path)
+	})
+
+	return value.(string), err
+}
+
+// execute sends a command function to the backend for execution. It returns 
+// the result received from the backend. This way all requests are searialized.
+func (t *topology) execute(cf commandFunc) (interface{}, error) {
+	cmd := newCommand(cf)
+
+	t.commandChan <- cmd
+
+	result := <-cmd.resultChan
+
+	return result.result, result.err
+}
+
+// backend manages the topology as a goroutine.
+func (t *topology) backend() {
+	for {
+		select {
+		case evt := <-t.zkEventChan:
+			// Change happened inside the topology.
+			if !evt.Ok() {
+				// TODO: Error handling, logging!
+			}
+
+			switch evt.Type {
+			case zookeeper.EVENT_CHANGED:
+				t.reload()
+			}
+		case cmd := <-t.commandChan:
+			// Perform the given command on the
+			// topology nodes.
+			cmd.perform(t.nodes)
+		}
+	}
+}
+
+// reload retrieves and parses the topology from ZooKeeper.
+func (t *topology) reload() error {
+	data, _, err := t.zkConn.Get("/topology")
+
+	if err != nil {
+		// TODO: Error handling, logging!
+		return err
+	}
+
+	if err = goyaml.Unmarshal([]byte(data), t.nodes); err != nil {
+		// TODO: Error handling, logging!
+		return err
+	}
+
+	return nil
+}
+
+// reset empties the topology.
+func (t *topology) reset() {
+	t.nodes = newTopologyNodes()
+}
+
+// dump returns the current topology as byte slice
+func (t *topology) dump() ([]byte, error) {
+	return goyaml.Marshal(t.nodes)
+}
+
+// EOF

=== added file 'state/util.go'
--- state/util.go	1970-01-01 00:00:00 +0000
+++ state/util.go	2011-12-20 10:52:07 +0000
@@ -0,0 +1,15 @@
+// launchpad.net/juju/state
+//
+// Copyright (c) 2011 Canonical Ltd.
+
+package state
+
+// --------------------
+// IMPORT
+// --------------------
+
+// --------------------
+// UTIL
+// --------------------
+
+// EOF

